package com.persagy.apm.energy.report.monthly.outline.service.builder.hotel.impl;

import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.dto.AddReportHotelProjectDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.*;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IHotelProjectReportPowerInfoOutlineService;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IHotelProjectReportPublicPowerInfoService;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IReportHotelProjectDetailService;
import com.persagy.apm.energy.report.monthly.outline.constants.HotelReportParagraphs;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IHotelProjectReportCostService;
import com.persagy.apm.energy.report.monthly.outline.service.builder.common.impl.ReportBuilder;
import com.persagy.apm.energy.report.monthly.outline.service.builder.hotel.IHotelProjectReportBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.List;

/**
 * 酒店项目报告建造者
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class HotelProjectReportBuilder extends ReportBuilder implements IHotelProjectReportBuilder {
    @Autowired
    private IReportHotelProjectDetailService reportHotelProjectDetailService;
    @Autowired
    private IHotelProjectReportCostService costService;
    @Autowired
    private IHotelProjectReportPublicPowerInfoService publicPowerInfoService;
    @Autowired
    private IHotelProjectReportPowerInfoOutlineService powerInfoOutlineService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean generateNewReportDetail(ReportOutline reportOutline) {
        try {
            AddReportHotelProjectDetailDTO addReportHotelProjectDetailDTO = new AddReportHotelProjectDetailDTO();
            // 获取项目费用信息
            addReportHotelProjectDetailDTO.setCostInfo(getCostInfo(reportOutline));
            // 获取项目用电量信息
            addReportHotelProjectDetailDTO.setPowerUsageOutline(getPowerConsumptionOutline(reportOutline));
            // 获取项目公区用电信息
            addReportHotelProjectDetailDTO.setPublicPowerUsage(getPublicPowerConsumption(reportOutline));
            // 获取项目冷站效率信息
            addReportHotelProjectDetailDTO.setAirConditionerEfficiency(getChillerPlanEfficiency(reportOutline));
            // 获取项目环境指标信息
            addReportHotelProjectDetailDTO.setEnvInfo(getEnvKpi(reportOutline));
            // 获取项目节能工单信息
            addReportHotelProjectDetailDTO.setEnergyWorkOrderInfo(getEnergyWorkOrderInfo(reportOutline));
            // 获取项目平台运行信息
            addReportHotelProjectDetailDTO.setPlatformRuntimeInfo(getMaintenanceInfo(reportOutline));
            // 获取项目报警处理信息
            addReportHotelProjectDetailDTO.setAlarmDealInfo(getAlarmDealInfo(reportOutline));

            String detailId = reportHotelProjectDetailService.createReportHotelProjectDetail(addReportHotelProjectDetailDTO);
            reportOutline.setDetailId(detailId);
            return true;
        } catch (Exception e) {
            log.error("生成报告失败", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    @Override
    public HotelProjectAlarmDealInfo getAlarmDealInfo(ReportOutline reportOutline) {
        // TODO: 2021/6/9
        return null;
    }

    @Override
    public HotelProjectMaintenanceInfo getMaintenanceInfo(ReportOutline reportOutline) {
        // TODO: 2021/6/9
        return null;
    }

    @Override
    public HotelProjectEnergyWorkerOrderInfo getEnergyWorkOrderInfo(ReportOutline reportOutline) {
        // TODO: 2021/6/9
        return null;
    }

    @Override
    public HotelProjectEnvKpi getEnvKpi(ReportOutline reportOutline) {
        // TODO: 2021/6/9
        return null;
    }

    @Override
    public HotelProjectChillerPlantEfficiency getChillerPlanEfficiency(ReportOutline reportOutline) {
        // TODO: 2021/6/9
        return null;
    }

    @Override
    public HotelProjectPublicPowerConsumption getPublicPowerConsumption(ReportOutline reportOutline) {
        List<String> functionIdList = getFunctionsByParagraph(
                reportOutline.getReportTypeId(), HotelReportParagraphs.PUBLIC_POWER_CONSUMPTION);
        return publicPowerInfoService.getPublicConsumption(reportOutline, functionIdList);
    }

    @Override
    public HotelProjectPowerConsumptionOutline getPowerConsumptionOutline(ReportOutline reportOutline) {
        List<String> functionIdList = getFunctionsByParagraph(
                reportOutline.getReportTypeId(), HotelReportParagraphs.POWER_CONSUMPTION_OUTLINE);
        return powerInfoOutlineService.getConsumptionOutline(reportOutline, functionIdList);
    }

    @Override
    public HotelProjectCostInfo getCostInfo(ReportOutline reportOutline) {
        List<String> functionIdList = getFunctionsByParagraph(
                reportOutline.getReportTypeId(), HotelReportParagraphs.COST);
        return costService.getCost(reportOutline, functionIdList);
    }
}
